// Shader: /Shaders/LightingModels/Phong.fsh
//

#version 110

#ifdef SHADOW
#extension GL_ARB_texture_rectangle : enable
#endif

struct LightSource
{
    vec3 Position;
    float Radius;
    
    //vec3 AmbientColor;
    vec3 DiffuseColor;
    vec3 SpecularColor;
};

struct Material
{
    vec3 DiffuseColor;
    float SpecularPow;
};

uniform LightSource u_LightSource;
uniform Material u_Material;

#ifdef SHADOW
uniform sampler2DRect u_TexShadow;
uniform float u_TexCoordScale;
#endif

varying vec3 out_N;
varying vec3 out_L;
varying vec3 out_V;

varying float out_Distance;

float CalcAttenuation( float d, float r )
{
    return clamp( 1.0 - pow( d / r, 2.0 ), 0.0, 1.0 );
}

float CalcDiffuse( vec3 N, vec3 L )
{
    return clamp( dot( N, L ), 0.0, 1.0 );
}

float CalcSpecular( vec3 N, vec3 L, vec3 V, float Pow )
{
    vec3 R = reflect( -L, N );
    
    return pow( clamp( dot( R, V ), 0.0, 1.0 ), Pow );  
}

vec4 CalcFragColor()
{
#ifdef SHADOW
    float Sh = texture2DRect( u_TexShadow, gl_FragCoord.xy * u_TexCoordScale ).a;
#endif
    float Att = CalcAttenuation( out_Distance, u_LightSource.Radius );
    
#ifdef SHADOW
    Att *= Sh;
#endif
    if (Att <= 0.0)
    {
        discard;
        return vec4( 0 ); // For compatibility with old NVIDIA Cg compiler.
    }
    else 
    {
        vec3 N = normalize( out_N );
        vec3 L = normalize( out_L.xyz );
        vec3 V = normalize( out_V.xyz );
        
        float Diffuse = CalcDiffuse( N, L );
        float Specular = CalcSpecular( N, L, V, u_Material.SpecularPow );   
    
        vec3 D = Diffuse * u_LightSource.DiffuseColor * u_Material.DiffuseColor;
        vec3 S = Specular * u_LightSource.SpecularColor;
             
        return vec4( Att * (D + S), 0.0 );
    } 
}

void main()
{
   gl_FragColor = CalcFragColor();
}