// Shader: /Shaders/Shadows/Wedge/Spherical/Coverage.fsh
//

#version 110

#extension GL_ARB_texture_rectangle : enable

//#define DISCARD
//#define ORIGINAL_SHADER

struct LightSource
{
    vec3 Position;
    vec3 WorldPosition;
    float Radius;
    float ScaledRadius;
    float InvRadius;
};

const float CosPiOverFour = 0.707106781187;
const float HalfInvPi = 0.5 * 0.3183099;

const vec3 InvZ = vec3( 1.0, 1.0, -1.0 );
const vec3 NegZ = vec3( 0.0, 0.0, -1.0 );

uniform LightSource u_LightSource;

uniform sampler2DRect u_TexCoordinate;
uniform samplerCube u_TexAtan2;

varying vec4 out_Edge0;
varying vec4 out_Edge1;

void CalcProjectedEdge( vec3 ShadedPoint, out vec3 Edge[2] )
{
    vec3 Dir = u_LightSource.WorldPosition - ShadedPoint;
    mat3 Mat;
    
    Mat[2] = normalize( Dir );
    Mat[0] = cross( Mat[2].yzx, Mat[2] );
    Mat[0] = normalize( Mat[0] );
    Mat[1] = cross( Mat[0], Mat[2] ); 
    
    Edge[0] = (out_Edge0.xyz - ShadedPoint) * Mat;
    Edge[1] = (out_Edge1.xyz - ShadedPoint) * Mat;
    
#ifdef ORIGINAL_SHADER
    float Scale = length( Dir ) * u_LightSource.InvRadius;
    // Transform into projected space.
    Edge[0].xy *= Scale;
    Edge[1].xy *= Scale;
#else
    float InvDist = 1.0 / length( Dir ); // inversesqrt( dot() ) ?
    
    Edge[0].xy *= u_LightSource.InvRadius;
    Edge[0].z *= InvDist;
    Edge[1].xy *= u_LightSource.InvRadius;
    Edge[1].z *= InvDist; 
#endif // ORIGINAL_SHADER      
}

float CalcCoverage()
{
    vec3 ShadedPoint = texture2DRect( u_TexCoordinate, gl_FragCoord.xy ).xyz;
    vec3 Edge[2];
    
    CalcProjectedEdge( ShadedPoint, Edge );
    
    vec3 Dir = Edge[1] - Edge[0];
    
#ifdef ORIGINAL_SHADER
    float a = dot( Dir * Dir, InvZ );
    float b = dot( Edge[0] * Dir, InvZ );
    b += b; // 2 * b 
    float c = dot( Edge[0] * Edge[0], InvZ );
    float D = b * b - 4.0 * a * c;
#else
    float a = dot( Dir * Dir, InvZ );
    float b = dot( Edge[0] * Dir, InvZ ); 
    float c = dot( Edge[0] * Edge[0], InvZ );
    float D = b * b - a * c;
#endif // ORIGINAL_SHADER
    
    if (D <= 0.0) 
    {
#ifdef DISCARD
        discard;
#else
        return 0.0;
#endif
    }
    else 
    {
        float Root = sqrt( D );
#ifdef ORIGINAL_SHADER
        vec2 t = vec2( -b - Root, -b + Root ) / (a + a);
#else
        vec2 t = vec2( -b - Root, -b + Root ) / a;
#endif // ORIGINAL_SHADER

		t = clamp( t, 0.0, 1.0 );		
        if (-Dir.z / length( Dir ) > CosPiOverFour)
            t[1] = 0.0;           
        
        vec3 Proj[2];
        
        Proj[0] = Dir * t[0] + Edge[0];
        Proj[0] = Proj[0] / Proj[0].z + NegZ;
        Proj[1] = Dir * t[1] + Edge[0]; 
        Proj[1] = Proj[1] / Proj[1].z + NegZ;
        
        vec2 Theta;
        
        Theta.x = textureCube( u_TexAtan2, Proj[0] ).x;
        Theta.y = textureCube( u_TexAtan2, Proj[1] ).x;
        
        float ArcArea = abs( Theta.x - Theta.y );
        if (ArcArea > 0.5)
            ArcArea = 1.0 - ArcArea;
                
        D = cross( Proj[0], Proj[1] ).z;
        float TriArea = abs( D ) * HalfInvPi; 
        float Coverage = ArcArea - TriArea;
        if (D > 0.0)
            Coverage = -Coverage;

        return clamp( Coverage, -1.0, 1.0 );
    }
}

void main()
{
    gl_FragColor.a = CalcCoverage(); 
}